#pragma GCC diagnostic ignored "-Wwrite-strings"
// Diese Zeile zum Ausblenden der diesbezueglichen Warn-Hinweise ist auch in der UTFT_SdRaw.cpp an oberster Stelle einzutragen.
// *******************************************************************************************************************************
// *** TFT9_50k.ino, DL2EBZ, 12.Nov.2018 *****************************************************************************************
// ***
/* Die Ansteuerung der Anzeige und die Einbindung der SD.Karte in diesem Programm basieren auf den Bibliotheken
   UTFT.h (v2.41) und der URTouch.h und von RinkyDinkElectronics   -->  web: http://www.RinkyDinkElectronics.com/ sowie der
   UTFT_SdRaw.h von Graham Lawrence  --> web: https://github.com/ghlawrence2000/UTFT_SdRaw , sowie der
   SdFAT.h von Bill Greimann --> web: https://github.com/greiman/SdFat
   Die UTFT_tinyFAT, als auch die tinyFAT v3.0 werden nicht benötigt !!!!!!
   Die Bild-Dateien muessen im Root-Ordner einer FAT16, oder FAT32 formatierten SDcard liegen, sowie im Karten-Slot des Display stecken.
   Hinweis: Aufgrund der 57"-TFT-Anzeige, muss das Bild-Format 800x480 Pixel sein.
          Fuer andere Formate kann ich auf den Beispiel-Sketch 'Demo_Landscape.ino aus der UTFT_TinyFAT verweisen.
   Um den 8-Seiten-Speicher des Display zu nutzen, kann der Sketch 'UTFT_CPLD_PageSwitching.ino' aus der UTFT-Lib als Basis genutzt werden.
   Wichtig ist in der Folge immer die Angabe des Chip-Satzes und die Pin-Belegung  --> UTFT myGLCD(ITDB50, 38, 39, 40, 41);  --> UTFT myGLCD(CTE50CPLD, 38, 39, 40, 41);  */
// ***
// *** Verbaut werden desweiteren DHT11, BMP085, DS3231 und Crius m8n
// *** Es werden zwei (!!) Serial-Ports angesprochen. Beide mit 9600 Baud
// *** Der Aufbau ist weitgehend 'modular' gestaltet. So gibt es eigenen void-Bereiche für diverse Aufgabenbereiche wie Sensorik und Darstellung
// ***
// *** Diese Version (_70c.ino) verzichtet auf Display und SD-Karte. Die Ausgabe der Einzelwerte erfolgt über den seriellen Monitor.
// *** Die Bereiche für Display und SD sind jedoch nur auskommentiert.
// ***
// *** erledigt: serielle Ausgabe von Datum, Zeit, Position, Monat, Wochentag
// ***           alle 30 Sekunden Ausgabe der Sensor-Werte
// ***           Sd-Karte mit TXT-Datei beschreiben, lesen, loeschen (ab Z.824), 09.11.2018
// ***           Uebernahme der Positionsdaten zur Locator-Berechnung
// ***           Uebernahme der Positionsdaten, sowie von Datum und ermittelter Tageszahl zur SA/SU-Berechnung
// ***           Script-Korrekturen fuer die Nutzung auch mit der aktuellen ARDUINO-IDE '1.8.6': char > const char; Time.h > TimeLib.h; Aenderung der UTFT_SdRaw.cpp
// ***           Synchronisation der RTC nach x-Tagen, sowie Anzeige der erfolgreichen Synchronisation (oder nicht), Dez.2018
// ***           Voll-Anzeige der RTC-Zeit bei fehlender GPS-Zeit, 17.Jan.2019
// ***           vorlaufende NULL bei RTC-Hauptanzeige fuer Sekunden und Minuten ist noch zu erstellen  --> 28.01.2019
// ***
// *** offen:    Log-Daten speichern
// ***           noch immer wird die Zahl nach der RTC-Synchronisation nicht unmittelbar korrigiert, bzw. ueberschrieben
// ***           Anzeige der Mond-Phase in %, passendes Bild einblenden, Anzeige der Sichtbarkeit in %
// ***           Mondbahn-Berechnung
// ***           Haupt-Anzeige der Zeit bleibt nur noch auf RTC
// ***           Piezo-Piepser Tick-Tack
// ***           weitere Default-Werte auf SD-Karte hinterlegen (immer anderen Dateinamen !) - z.B. Positionsdaten
// ***           "Blink" geht seit TFT9_50l.ino - also seit der Hinzunahme der Mondberechnung - nicht mehr


// *** fuer Display & SD-Karte ***
#include <SdFat.h>
#include <UTFT_SdRaw.h>
#include <UTFT.h>
#include <URTouch.h>
#include <math.h>

// UTFT myGLCD(CPLD,38,39,40,41);
// UTFT myGLCD(CTE50CPLD, 38, 39, 40, 41);    // angepasste Version für den 16-bit-Chipsatz MD090SD
UTFT myGLCD(ITDB50, 38, 39, 40, 41); // für 5", Touch, SD, mit SSD1963
URTouch  myTouch(6, 5, 4, 3, 2);
UTFT_SdRaw myFiles(&myGLCD);

// *** Bestimmung der eingesetzten Schriftarten ***
extern uint8_t SmallFont[];
extern uint8_t BigFont[];
extern uint8_t SevenSegNumFont[];
extern uint8_t SevenSeg_XXXL_Num[];

const uint8_t chipSelect = 53; // *** SD chip select pin ***
SdFat sd;                      // *** file system ***
File syncFile;
int SYNC = 0;                  // default-Wert
int moon = 0;                  // default-Wert
int dastr = 0;                 // default-Wert

// *** fuer Sensorik ***
#include "DHT.h"
#define DHTPIN  10                  // Temperatur-Sensor
#define DHTTYPE DHT11               // DHT11, DHT21, DHT22 
DHT dht(DHTPIN, DHTTYPE);

#include <Wire.h>
#include <Adafruit_BMP085.h>
Adafruit_BMP085 bmp;

// *** fuer die LOG-Datei ***
File logDat;                        // diese Variable 'logDat' im Format 'File' nimmt spaeter den Dateinamen und die Befehle auf
String dataString = "";             // fuer die Aufnahme sämtlicher Sensor-Daten
int go = 0;                         // SD-Schreib-Freigabe



//------- Datum  Uhrzeit  Sommer-/Winterzeit ------------------------------------------------------------------------
#include "RTClib.h"                  // RTC-Bibliothek einbinden
RTC_DS1307 rtc;                      // ggf.RTC , RTC Modul
#include <TimeLib.h>                 // Anpassung wg. neuer ARDUINO-EDI > alt:  #include <Time.h>
#include <Timezone.h>
TimeChangeRule CEST = {"MESZ", Last, Sun, Mar, 2, 120};
TimeChangeRule CET =  {"MEZ", Last, Sun, Oct, 3, 60};
// TimeChangeRule CEST = {"AEST", First, Sun, Mar, 2, 600};   // Beginn der Sommerzeit UTC +10 Std. AustralinDaylightTime
// TimeChangeRule CET  = {"AEDT", First, Sun, Oct, 3, 660};   // Beginn der Winterzeit UTC +11 Std. AustralianEasternStandardTime

Timezone CE(CEST, CET);              // Die Variable 'CE' beinhaltet die Info, in welcher Zeitzone wir uns aktuell befinden
TimeChangeRule *tcr;
time_t utc, local;                   // zusaetzlich
char *Tag[7] = {"SONNTAG", "MONTAG", "DIENSTAG", "MITTWOCH", "DONNERSTAG", "FREITAG", "SONNABEND"};
char *Monat[12] = {"Januar", "Februar", "Maerz", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"};
String Foto;

int UTC;
int diff;

String Lat = " "; // Latitude
String La1 = " ";
String Lon = " "; // Longitude
String Lo1 = " ";

int s10 = 0, s01 = 0; // Definition Sekunde
int m10 = 0, m01 = 0; // Definition Minute
int h10 = 0, h01 = 0; // Definition Stunde
int u10 = 0, u01 = 0; // Definition UTC-Stunde

// ** Sekunden-Tick **
#define TICKPIN  8  // Pin für den Ton-Ausgang an MEGA-2560 , bei UNO TICKPIN = 3

String Line = "";           // eine String-Variable für das halten von Eingangsdaten
unsigned long nogps = 1000; // Wartezeit nach dem Start für das Eingangs-Signal vom Empfaenger
int Signal = 0;             // Variable zur Anzeige ob SAT-Stunden-Empfang vorhanden ist - um darüber auch die RTC auszugeben, oder nicht
int ZAHL = 0;               // umkonvertierter Wert aus der SD-Datei


// ********* Sonnenberechnungen ******************
int result = 0;
double pi2 = 6.283185307179586476925286766559;
double pi  = 3.1415926535897932384626433832795;
double RAD = 0.017453292519943295769236907684886;
double laenge, breite;
char breiterichtung, laengerichtung;
double aB, aC, aD, Deklination, Sonnenaufgang, Sonnenuntergang, aF, aG;
double hs = 0.0145;                      // double hs = -50.0 / 60.0 * RAD; // Wert der Horizonthoehe
double ZD, Zw1, Zw2;                     // Wert der Zeitdifferenz
double ZG, ZG1;                          // Dezimal-Wert der Zeitgleichung, ZG1 Wert in Minuten
double cAZ1, cAZ2, cAZ3, cAZ4, cAZ, AZ1, AZ2, AZ3, AZ4, AZ  ;      // Azimut, Sonnenhoehe in Grad
float SW, SW1, SW2, SW3, SWm;            // Stundenwinkel
float SH, SH1, SH2, SH3, SH4, SH5;       // aktuelle Sonnenhoehe
float SHm, SHm1;                         // Mittagshoehe
int Tg = 82;                             // Anzahl Tage im lfd. Jahr  // 81 = 20.03. - Tag-/Nachtgleiche
int doy, stunde, minut;                  // Day of the year - Tageszahl in lfd. Jahr
double WOZ, MOZ, MOZ2;                   // wahre Ortszeit, mittlere Ortszeit, dto. jedoch für SU
double SAm, SA, SUm, SU, SAmin, SUmin;   // (mittlere / tatsaechliche) Sonnenaufgangs-, Sonnenuntergangszeiten
double TgLS1, TgLS2, TgLS3, TgLS4, TgLS5;// Tageslaenge Stunde - Berechnung
int TgLS, TgMS, TgLM, TgMM;              // Tageslaenge Ergebnis (Stunde, Minute, TgMitteStunde, TgMitteMinute)
int MOZStd, SAMinute, MOZStdU, SUMinute; // Zahlenwert für die SA/SU Stunden und Minuten
double Wert;
int SZ;


// ********* Maidenhead-Locator *******************
double B, L;
int Loc1, Loc2, Loc3, Loc4, Loc5, Loc6;
char* Zeichen[] = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"};


// ********* unabhaengiges Blinken der Anzeige *** ohne delay() *********
// ** http://playground.arduino.cc/Learning/BlinkWithoutDelay_de
unsigned long previousMillis = 0; // speichert wie viele Sekunden seit derletzten Aenderung vergangen sind
unsigned long interval = 1250;    // Interval zwischen zwei Aenderungen
boolean value = LOW;              // Startwert

char ba[10]; // Baro
char l1[10]; // Thermo
char b1[10]; // Hygro

// ************************************************************************************
// ***** S E T U P *****
// ************************************************************************************
void setup()
{
  myGLCD.InitLCD();
  // myTouch.InitTouch();
  Serial.begin(9600);
  Line.reserve(100);         // 100 Zeichen pro Zeile

  if (!sd.begin(chipSelect, SPI_FULL_SPEED)) sd.initErrorHalt();
  Serial.println("Karte wurde initialisiert.");

  // **** DHT11-Sensor starten ****
  dht.begin();                   // DHT-Sensorik

  // **** BMP085-Daten starten ****
  if (!bmp.begin())
  {
    Serial.println("Kein BMP085 gefunden ... I2C Adresse und Verkabelung pruefen !");
    while (1) {}
  }

  // ******** Anzeige des home-screen ********
  // Serial.println("Aufruf Home-Screen");
  // screenHome();  // eigener Funktionsbereich
  myGLCD.clrScr();                            // clear the screen
  myGLCD.setColor(255, 255, 255);             // schwarzer Hintergrund
  myFiles.load(0, 0, 800, 480, "qmk804.raw"); Foto = "qmk804.raw"; // Quermarkenfeuer, 800x480 Pixel-Foto laden
  // myFiles.load(0, 0, 800, 480, "PIC805.raw"); Foto = "PIC805.raw"; // Planet, 800x480 Pixel-Foto laden
  // myFiles.load(480, 30, 62, 62, "3_Mond.raw"); Foto = "3_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  delay(2000);  // einmalig etwas Pause vor dem Start weiterer Funktionen


  // **** GPS-Daten-Empfang pruefen ***************
  Serial1.begin(9600);  // Start der seriellen Komunikation mit dem GPS-Empfaenger
  {
    while (Serial1.available())Serial1.read();  // Pruefen ob GPS-Daten vorhanden --> RX1
    Serial.println("RX1-Daten werden empfangen (GPS). ");
  }
  int h = 23; int m = 59; int s = 45;
  setTime(h, m, s, 0, 0, 0); // das Ergebnis ist dann der 31.12.1999, 23:59:45 (SOON 2K)


  // **** RTC in Betrieb nehmen *************
  // *** Es gibt hier die Moeglichkeit die Uhrzeit fix vorzugeben *** Hier beispielhaft 22. Mai 2018, 10:23:00 Uhr **** Dazu ist die Folgezeile einzukommentieren. ****
  // RTC.adjust(DateTime(2018, 6, 24, 18, 40, 00));
  // *** ... und nach dem Hochladen ist dann diese Zeile wieder auszukommentieren +++++++ !!! .. und dann nochmals hochladen !!!!! ***
  DateTime compiled = DateTime(__DATE__, __TIME__); // Zeit von PC holen
  rtc.begin();                 // Initialisiere RTC
  if (! rtc.isrunning())
  {
    Serial.println("RTC laeuft nicht !");
    // nichts weiter machen
  }


  // *** unteren Uhrzeit-Balken erstellen ******************
  // myGLCD.setColor(122, 122, 122);     // grau - untere Zeitanzeige
  // myGLCD.fillRect(35, 340, 655, 460); // x1, y1, x2, y2,  --> 35, 340, 655, 470


  // *** Auslesen des Datei-Inhalt von der SD  *** --> https://stackoverflow.com/questions/51309772/from-txt-file-in-sd-card-to-string-variable-in-arduino
  syncFile = sd.open("sync.txt");       // oeffne die Datei wieder zu Testzwecken
  if (syncFile)
  {
    String WERT = "";
    while (syncFile.available())        // wenn die Datei vorhanden ist: lies (Bit-weise) aus der Datei
    {
      WERT += ((char)syncFile.read());  // schreibe ein Zeichen nach dem anderen in die String-Variable
    }
    syncFile.close();                   // schliesse die Datei
    // Serial.print("Der Wert lautet: "); Serial.println(WERT); // Gebe die String-Variable aus
    ZAHL = atoi(WERT.c_str());          // konvertiere von String zu Int
    Serial.print("Die Zahl lautet: "); Serial.println(ZAHL); // Gebe die Int-Variable aus
  }
  else
  { // falls sich die Datei nicht oeffnen laesst, gebe eine Fehlermeldung raus
    Serial.println("Fehler beim Oeffnen der sync.txt"); // Serial.println("sync.txt wurde geloescht.");
  }
} // **** Ende SETUP *****



// ****************************************************************************************************
// ***** L O O P *****
// ****************************************************************************************************
void loop()
{
  // **** Sensorik alle 30 Sekunden aufrufen ****
  if (second() == 45 || second() == 15)  // alle 30 Sekunden (war zuvor ==30 || ==0)
  {
    screenSens();  // ** Aufruf der Sensorik
  }
  else
  {
    // nichts tun
  }  // *** Ende Sensorik ***

  // Serial.print("Millis= "); Serial.println(millis());
  blinker();      // ** Aufruf des Leuchtturm-Blinkers


  // *** G P S ***********************************************************************
  static int os = -1, om = -1, oh = -1;
  if (!(os % 3))if (getline("$GNGGA"))GGA();  // wenn os nicht gefunden wurde
  if (!(os % 5))if (getline("$GNRMC"))RMC();
  SerialClear();

  if (second() != os)              // ** jede Sekunde **
  {
    s01 = second() % 10; s10 = second() / 10;
    m01 = minute() % 10; m10 = minute() / 10;
    h01 = hour()   % 10; h10 = hour()   / 10;
    u01 = UTC      % 10; u10 = UTC      / 10;
    beep();   // Stundensignal
    // Serial.println("Tick");      // Hier würde schoen eine Schleife passen, die dann noch jede 2.Sekunde 'Tack' ausgibt

    char t[10];
    char std[2]; // local-Stunde
    char m[2];   // local-Minute
    char u[2];   // UTC-Stunde

    if (om != minute())             // ** jede Minute wird auch jede Minute ausgegeben **
    {
      //  tft.setTextSize(3); tft.setTextColor(RED,BLACK);
      if (oh != hour())              // ** jede Stunde wird nur jede Stunde ausgegeben  **
      {
        oh = hour();
      } // *** Ende Stunde ***

      // **** Ausgabe auf den seriellen Monitor ****
      Serial.print("  Uhrzeit: ");
      Serial.print(char('0' + h10));         // erst die 10er-Stelle der Stunde
      Serial.print(char('0' + h01));         // dann die 1er-Stunde
      Serial.print(':');                     // dann der Doppelpunkt
      Serial.print(char('0' + m10));         // dann die 10er-Minute
      Serial.println(char('0' + m01));       // und dann die 1er-Minute

      // *** Stunden-Anzeige und dessen Umrechnung sowie Anzeige ***
      Serial.print(" Zeitzone: ");   Serial.print(tcr -> abbrev); Serial.print(" "); Serial.print(hour()); Serial.println(".Stunde"); // Anzeige der lokalen Zeitzone
      Serial.print("        UTC ");  Serial.print(char('0' + u10)); // erst die 10er-UTC
      Serial.print(char('0' + u01)); Serial.println(".Stunde");     // und dann die 1er-Anzeige der UTC-Stunde

      myGLCD.setFont(BigFont);               // Schrift-Fonts
      myGLCD.setColor(124, 252, 0);          // Schriftfarbe: gruen
      myGLCD.print(tcr -> abbrev, 105, 423); // Anzeige der lokalen Zeitzone (MEZ, MESZ), Spalte, Zeile

      // *** Wochentag                  *** wird jede Minute ausgegeben ***
      if (year() < 2018)                 // ohne GPS-Empfang in Anzeige in gelb, ...
      {
        myGLCD.setColor(VGA_YELLOW);     // Schriftfarbe: gelb
      }
      else                               // ... sonst gruen
      {
        myGLCD.setColor(124, 252, 0);    // Schriftfarbe: gruen
      }

      { char s[20] = " ";
        strcat(s, Tag[weekday() - 1]);
        strcat(s, "   ");
        Serial.print("Wochentag:"); Serial.println(s);
        myGLCD.print(s, 110, 450);           // .. Spalte 110 (rechts/link), Zeile 450 (oben/unten)
      }


      // *** Datum                      *** wird jede Minute ausgegeben ***
      { char s[20];
        sprintf(s, "%02d.%03s.%04d", day(), monthShortStr(month()), year()); // Aufbau einer einzigen Variablen mit Tag, Monat und Jahr
        Serial.print("    Datum: "); Serial.println(s);
        myGLCD.print(s, 285, 450);           // .. Spalte 285, Zeile 450
      }

      { // *** ausgeschriebener Monatsname *** wird jede Minute ausgegeben ***
        char s[20] = " ";
        strcat(s, Monat[month() - 1]);
        strcat(s, "   ");
        Serial.print("    Monat:"); Serial.println(s); Serial.println();
        myGLCD.print(s, 465, 450);           // .. Spalte 465, Zeile 450
      }

      om = minute();


      // **********************************************************************************
      // **** Position ****** Maidenhead-Locator ******************************************
      // **********************************************************************************
      // die Standort-Koordinaten (laenge, breite) stammen aus obigen 'Sonnenberechnung'
      // Bei dieser Thematik hat mich Carsten, 'DL8AAP' unterstützt und mir die Grund-Formel besorgt.
      // Die String-Variablen 'Lat', 'Lon', ... sind in die Int-Variablen 'leange' und breite umzuwandeln
      // Zuvor werden sie in die Einzelbestandteile zerlegt. Beispiel:  *** Luegde ***
      // laenge = 9.2433177;   //  9°14'24.52"   // laengerichtung = 'E';
      // breite = 51.961246;   // 51°57'66.89"   // breiterichtung = 'N';
      // Laengengrad = E 9°14'24"  Breitengrad = N 51°57'66"
      // Die Daten werden zunaechst in lg=9° lm=14' ls=25" lr=E  und bg=51° bm=57' bs=67" br=N zerlegt :


      // *** Ausgabe der Positionswerte ** diese Daten sind noch unberechnete Teil-Strings aus dem GPS-Empfang ********************
      myGLCD.setColor(VGA_WHITE);           // Schriftfarbe
      myGLCD.print(Lat,  70, 60); myGLCD.print(La1, 253, 60); // 34, 60 + 217, 60
      myGLCD.print(Lon,  56, 80); myGLCD.print(Lo1, 253, 80); // 20, 80 / 217, 80
      Serial.print("Latitude:   "); Serial.print(Lat); Serial.print(" "); Serial.println(La1);  // geografische Breite --> Latitude
      Serial.print("Longitude: ");  Serial.print(Lon); Serial.print(" "); Serial.println(Lo1);  // geografische Länge  --> Longitude

      // *** Positionsdaten in Grad, Minute und Sekunde zerlegen **************************
      // **********************************************************************************
      //  ********* B r e i t e n g r a d ******************
      //  **************************************************
      // *** Umwandlung der STRING-Variablen fuer die BREITE in einzelne int-Variablen (int: ohne Nachkommastellen). Das Ergebnis von 5157.66779 lautet 5157
      long iLat = Lat.toInt();
      // Serial.print("iLat = "); Serial.println(iLat);

      // *** Umwandlung der STRING-Variablen fuer den Breitengrad (float: mit 2 Nachkommastellen). Das Ergebnis von 5157.66779 lautet 5157.67
      float fLat = Lat.toFloat();
      int bg = (fLat / 100);                  // Breitengrad      --> 51°
      // Serial.print("Breitengrad bg= "); Serial.println(bg);

      int bm = (fLat - (bg * 100));           // Breitenminute    --> 57'
      // Serial.print("Breitenminute bm= "); Serial.println(bm);

      int bs = ((fLat * 100) - (iLat * 100)); // Breitensekunde   --> 40"
      // Serial.print("Breitensekunde bs= "); Serial.println(bs);


      // *** Positionsdaten in Grad, Minute und Sekunde zerlegen **************************
      // **********************************************************************************
      //  ********* L a e n g e n g r a d ******************
      //  **************************************************
      // *** Umwandlung der STRING-Variablen fuer die LAENGE in einzelne int-Variablen (int: ohne Nachkommastellen). Das Ergebnis von 914.24855 lautet 914
      long iLon = Lon.toInt();
      // Serial.print("iLon = "); Serial.println(iLon);

      // *** Umwandlung der STRING-Variablen fuer den Laengengrad (float: mit 2 Nachkommastellen). Das Ergebnis von 914.24855 lautet 914.25
      float fLon = Lon.toFloat();
      int lg = (fLon / 100);                  // Breitengrad      -->  9°
      // Serial.print("Laengengrad lg= "); Serial.println(lg);

      int lm = (fLon - (lg * 100));           // Breitenminute    --> 14'
      // Serial.print("Laengenminute bm= "); Serial.println(lm);

      int ls = ((fLon * 100) - (iLon * 100)); // Breitensekunde   --> 25"
      // Serial.print("Laengensekunde bs= "); Serial.println(ls);



      // ******* Berechnungen ************
      // *********************************
      // Serial.print("bs = "); Serial.println(bs);
      if (bs == 0)  // Nur wenn die Variable bs nicht '0' ist, sollen die weiteren Berechnungen stattfinden
      {
        Serial.println("Noch ist keine zuverlaessige Positionsangabe verfuegbar.");
      }
      else
      { // Serial.println("Positionswert ist vorhanden.");
        // *** fuer die BREITE **** Die Formel lautet: breite = bg + (bm/60) + (bs/3600) = 51,5767260
        // Serial.print("Breiterichtung = "); Serial.println(breiterichtung);
        double brei1 = ((double)bm / 60);   // hier wird die int-Variable 'bm' als double-Variable gerechnet   // = 0,95
        // Serial.print("brei1= ");Serial.println(brei1);
        double brei2 = ((double)bg + brei1);   // = 51,95
        // Serial.print("brei2= ");Serial.println(brei2);
        breite = (brei2 + ((double)bs / 3600)); // -->  51 + 57/60 + 40/3600 = 51,97
        // Serial.print("Die 'dezimale Breite' betraegt= "); Serial.println(breite);

        // *** fuer die LAENGE **** Die Formel lautet: laenge = lg + (lm/60) + (ls/3600) =  9,14253611
        // Serial.print("Laengerichtung = "); Serial.println(laengerichtung); Serial.println("");
        double laen1 = ((double)lm / 60);       // = 0,14
        // Serial.print("laen1= "); Serial.println(laen1);
        double laen2 = ((double)lg + laen1);    // = 9,14
        // Serial.print("laen2= "); Serial.println(laen2);
        laenge = (laen2 + ((double)ls / 3600)); // = 9,1469
        // Serial.print("Die 'dezimale Laenge' betraegt= "); Serial.println(laenge);


        // Die String-Variablen 'Lo1' (= N) und 'La1' (=E)  werden von STRING zu char umgewandelt.
        char breiterichtung[2];
        La1.toCharArray(breiterichtung, 2);  // Ost oder West

        char laengerichtung[2];
        Lo1.toCharArray(laengerichtung, 2);  // Nord oder Sued

        if (Lo1 == "S")   // Die char-Variablen dienen nun zur Bestimmung der Multiplikatoren
        {
          breite = breite * -1;
          // Serial.println("breiterichtung ist S");
        }
        else
        {
          breite = breite;
          // Serial.println("breiterichtung ist N");
        }

        if (Lo1 == "W")
        {
          laenge = laenge * -1;
          // Serial.println("laengerichtung ist W");
        }
        else
        {
          laenge = laenge;
          // Serial.println("laengerichtung ist E"); Serial.println("");
        }

        // ** Berechnung des Breiten-Locators **
        // *************************************
        B = ((breite + 90) / 10);     // (51,97 + 90) / 10 = 14
        Loc2 = int(B);                // 14
        B = ((10 * B) - (10 * Loc2)); // (10 * 51,97) - (10 * 14) = 379,7
        Loc4 = int(B);                // 1
        String Lo4 = String(Loc4);    // Umwandlung von INT zu STRING für Display-Anzeige
        B = ((24 * B) - (24 * Loc4)); // 23,0685333
        Loc6 = int(B);                // 23

        // ** Berechnung des Laengen-Locators **
        // *************************************
        L = (laenge + 180) / 20;    // 9,461875
        Loc1 = int(L);              // 9
        L = (10 * L) - (10 * Loc1); // 4,61875
        Loc3 = int(L);              // 4
        String Lo3 = String(Loc3);    // Umwandlung von INT zu STRING für Display-Anzeige
        L = (24 * L) - (24 * Loc3); // 14,85
        Loc5 = int(L);              // 14

        // ** kompletten Locator-String bilden ***
        // ***************************************
        // Dazu muß die Position des einzelnen Buchstaben ermittelt werden. Dazu wird jeweils die zuvor ermittelte Zahl einem Buchstaben zugeordnet.
        char* Zeichen[] = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"};
        Serial.print("Der Locator lautet: ");             Serial.print(Zeichen[Loc1]); Serial.print(Zeichen[Loc2]); Serial.print(Loc3);
        Serial.print(Loc4); Serial.print(Zeichen[Loc5]); Serial.println(Zeichen[Loc6]);
        myGLCD.print("Locator: ", 34, 100); myGLCD.print(Zeichen[Loc1], 200, 100); myGLCD.print(Zeichen[Loc2], 215, 100); myGLCD.print(Zeichen[Loc5], 260, 100); myGLCD.print(Zeichen[Loc6], 275, 100); myGLCD.print(Lo3, 230, 100); myGLCD.print(Lo4, 245, 100);
      } // ***** Ende Locator-Berechnung ***************************************************************************************




      // ***************************************************************************
      // ******* Sonnenberechnungen *****************************
      if (bs == 0)  // Nur wenn die Variable bs nicht '0' ist, sollen die weiteren Berechnungen stattfinden
      {
      }
      else
      {
        // result = calculateDayOfYear(26, 3, 2018);         // nur zu Testzwecken
        result = calculateDayOfYear(day(), month(), year()); // mit Tagesdatum
        // Serial.print("Diff: "); Serial.println(diff);
        if (diff == 2) // wenn die Zeitdifferenz zwischen der CET und der UTC = 2 ist, dann ...
        {
          SZ = 2;
          // Serial.println("Die errechnete Zeitdifferenz 'diff' ergibt:  M E S Z ");
          //(x-start, y-start, breite, Hoehe, Farbe);
          //(Spalte, Zeile, Breite, Hoehe, Farbe);
          // tft.fillRect(245, 95, 57, 25, 0x07E0);   // male den Bereich für die Zeitzone gruen aus
          // tft.setTextSize(2); tft.setCursor(250, 100); tft.print("MESZ");
        }
        else
        {
          SZ = 1;
          // Serial.println("Die errechnete Zeitdifferenz 'diff' ergibt: M E Z ");
          //(Spalte, Zeile, Breite, Hoehe, Farbe);
          // tft.fillRect(245, 95, 57, 25, 0x07E0);  // male den Bereich für die Zeitzone gruen aus
          // tft.setTextSize(1); tft.setCursor(255, 100); tft.print("MEZ");
        }

        {
          // ******* Radiant der Breite *****
          aB = pi * breite / 180;                         // stimmt = 0.90722217 bei 51.98 Breite
          Serial.print("Der Radiant der Breite 'aB'= "); Serial.println(aB);

          // ******* Deklination ********** --> das ist der Winkel, den die Sonne von der Senkrechten (90°) ueber dem Aequator abweicht = -23.5° bis (+)23,5°
          aC = 57.29578 * (0.016906 * (result - 80.086)); //  - 74 ist die Tageszahl fuer den 15.03.= -48.51542608527448
          aD = sin(aC * pi / 180);                        //  -  0.749134094 --> es muss zusaetzlich noch auf Bogenmaß umgerechnet werden ***
          Deklination = 0.4095 * aD;                      //     0.3067704118
          aF = 360 / (2 * pi) * Deklination;              //  Umrechnung von Bogenmass zu Gradzahl
          // Serial.print("Deklination= "); Serial.print(Deklination); Serial.print(" .. und als Gradzahl 'aF'= "); Serial.println(aF);
          Serial.print("  Deklination= "); Serial.println(aF);

          // ******** Zeitdifferenz ******
          ZD = 12 * acos ((sin(-hs) - sin(aB) * sin(Deklination)) / (cos(aB) * cos(Deklination))) / pi;
          Serial.print("Zeitdifferenz=  "); Serial.println(ZD);
          Zw1 = acos (sin(-hs) - sin(aB) * sin(Deklination)); // erster Wert der Zeitdifferenz
          Zw2 = (cos(aB) * cos(Deklination)) / pi;            // zweiter Wert der Zeitdifferenz

          // ******** Zeitgleichung ******
          ZG = -0.171 * sin(0.0337 * result + 0.465) - 0.1299 * sin(0.01787 * result - 0.168); // Differenz = wahre Ortszeit - mittlere Ortszeit als Dezimalwert
          ZG1 = ZG * 60; // Differenz in Minuten
          Serial.print("Zeitgleichung= "); Serial.print(ZG); Serial.print("  .. und in Minuten: "); Serial.println(ZG1);

          // ******* Stundenwinkel *******
          if (diff == 2)  // hier die Korrektur fuer die MESZ, da die eingesetzte Formel diese Variable nicht beruecksichtigt
          {
            stunde = (oh - 1);
            // Serial.print("die korrigierte Stunde lautet= "); Serial.println(stunde);
          }
          else
          {
            stunde = oh;
          }
          minut  = om;
          SW = ((((stunde * 60) + minut + ZG1 + (4 * laenge) - 60) / 4) - 180);  // ** Das Setzen der Klammern ist die Schwierigkeit **
          Serial.print("Stundenwinkel=  "); Serial.println(SW);

          // **** Sonnenhoehe ** Vertikal-Winkel *******
          SH4 = ((sin(((pi / 180) * breite)) * (sin((pi / 180) * aF))) + (cos(((pi / 180) * breite)) * (cos((pi / 180) * aF) * cos((pi / 180) * SW))));  // 0,46 als Bogenmass
          SH1 = (asin(SH4) / (pi / 180));     // entsprechend 26,97°  *** 'asin' erwartet eine Grad-Zahl und kein Bogenmass; daher (.. / (Pi/180) !!! ***


          // ***** Azimut ** Horizontal-Winkel ** Kompass-Kurs **********
          cAZ1 = (sin((pi / 180) * breite) * SH4);               // 1.Teil der Formel --> sinus(0,01745329 *Laenge) * Wert'sinus_Sonnenhoehe'
          cAZ2 = (sin((pi / 180) * aF));                         // 2.Teil der Formel --> sinus(Radiant der Breite * Deklination)
          cAZ3 = (cAZ1 - cAZ2);                                  // 3.Teil der Formel --> Teil1 - Teil2
          cAZ4 = (cos((pi / 180) * breite) * (sin(acos(SH4))));  // 4.Teil der Formel --> (cosinus(Radiant der Breite * Laenge)) * (sinus(arcossinus(sinus_Sonnenhoehe))
          cAZ  = ((cAZ3 / cAZ4) * -1);                           // 5.Teil der Formel --> Teil3 / Teil4
          Serial.print("  cos(Azimut)=  "); Serial.println(cAZ); // Das Ergebnis stimmt !


          //  ** Umrechnung vom Bogenmass auf die Grad-Zahl ********
          float m3 = minut; float m4 = m3 / 60;             // der Wert der 'minut' ist umzukonvertieren
          AZ1 = (stunde + m4);                              // Vorgabe --> (stimmt) =stunde + umkonvertierte !!!! (Minute/60)
          AZ4 = ((12 + ((15 - laenge) / 15) - (ZG1 / 60))); // Vergleichswert (stimmt) --> =12+(15-laenge)/15-ZG1/60
          // Serial.print("AZ1= "); Serial.print(AZ1); Serial.print("  AZ4= "); Serial.println(AZ4);
          if (AZ1 <= AZ4)                                   // Vergleich, um ein hoeheres Bogenmass als 360° auszuschliessen
          {
            AZ = (acos(cAZ) / (pi / 180));                  // ARCCOS(L29)/$B$9  --> Formel stimmt
          }
          else
          {
            AZ = 360 - acos(cAZ) / (pi / 180);              // 360-ARCCOS(L29)/$B$9) --> Formel stimmt
          }

          // ** Berechnung der Zeitdifferenz und SA, sowie SU *******
          Sonnenaufgang = 12 - ZD;         // wahrer Sonnenaufgang
          MOZ = Sonnenaufgang + (-1 * ZG); // Sonnenaufgang auf mittlerer Ortszeit; da der Variablen-Wert negativ ist, muß der erst noch umgerechnet werden

          WOZ = -0.171 * sin(0.0337 * result + 0.465) + ZG;    // = wahre Ortszeit
          SAm = 12 - ZD - ZG;                                  // = Sonnenaufgang, mittlere Zeit  (MOZ)
          SUm = 12 + ZD - ZG;                                  // = Sonnenuntergang, mittelere Zeit (MOZ)

          // ***** jetzt noch die Laengengrad-Abweichung, sowie die Zeit-Abweichung von UTC zu MEZ, bzw. MESZ ****
          // ******** Sonnenaufgang *********
          SA = MOZ + (-laenge / 15) + SZ;    // SZ= Stundenabweichung von 'GMT', bzw. +1 entspricht MEZ; +2 ist MESZ
          // *** es folgt die Umrechnung der Nachkommastelle, bzw. Minuten-Zahl ***
          MOZStd    = SA;                    // nur die Ganzzahl - vor dem Komma
          SAmin     = SA - MOZStd;           // nur die Nachkommazahl
          SAMinute  = SAmin * 60;            // Umrechnung auf Minute

          // ******** Sonnenuntergang ********
          Sonnenuntergang = 12 + ZD;
          MOZ2     = Sonnenuntergang + (-1 * ZG);
          SU       = MOZ2 + (-laenge / 15) + SZ; // SZ= Stundenabweichung von 'GMT', bzw.: +1 entspricht MEZ; +2 ist MESZ
          MOZStdU  = SU;
          SUmin    = SU - MOZStdU;
          SUMinute = SUmin * 60;

          // ********* Tageslaenge ************
          TgLS1 = ((MOZStd * 60)  + SAMinute); // Umrechnung des SA auf Minuten
          TgLS2 = ((MOZStdU * 60) + SUMinute); // Umrechnung des SU auf Minuten
          TgLS3 = (TgLS2 - TgLS1);             // Differenz von beiden Vorwerten
          TgLS  = (TgLS3 / 60);                // Ergebnis durch 60 teilen und als Ganzzahl (int) ausgeben
          TgLM  = (TgLS3 - (TgLS * 60));       // Restsumme der Minuten nach Abzug der Stunden-Minuten

          // ********** Uhrzeit des hoechsten Sonnenstandes ****
          TgLS4 = (TgLS3 / 2);                 // halbe Tageslaenge
          TgLS5 = (((MOZStdU * 60) + SUMinute) - TgLS4);  // Hoechststand in Minuten -- Untergangszeit - halbe Tageslaenge
          TgMS  = (TgLS5 / 60);                // Hoechststand-Stunde
          TgMM  = (TgLS5 - (TgMS * 60));       // Hoechststand-Minute

          // ******** Grad-Zahl des hoechsten Sonnenstandes ****
          // Wenn die Mittagszeit bekannt ist, dann kann eine Simulationsrechnung auf die Hoehe erstellt werden.
          // Dazu muß zunaechst der Stundenwinkel des Hoechststandes errecht werden. Die anderen Formel-Werte bleiben gleich.
          // Der Azimut-Winkel ist zum Hoechststand immer 180°.
          // SHm, SH1m, SWm;   .... aF = Deklination   ZG1 = Zeitgleichung  .. SW = Stundenwinkel (muss neu berechnet werden)
          SWm = (((((TgMS - SZ + 1) * 60) + TgMM + ZG1 + (4 * laenge) - 60) / 4) - 180); // die beginnende Stundenzahl 'TgMS' ist sher kryptisch - resultiert aber aus der Korektur von MEZ zu MESZ mit der bereits vorhandenen Variablen 'SZ'
          SHm1 = ((sin(((pi / 180) * breite)) * (sin((pi / 180) * aF))) + (cos(((pi / 180) * breite)) * (cos((pi / 180) * aF) * cos((pi / 180) * SWm))));
          SHm = (asin(SHm1) / (pi / 180));     // Umrechnung von Bogenmass auf Grad-Zahl
        } // ** Ende Sonnenberechnung ***

        // *** Ausgabe von SA & SU auf Display und 'seriellen Monitor' ************************************
        Serial.print("           SA=  "); Serial.print(MOZStd);  Serial.print(":"); Serial.println(SAMinute);
        Serial.print("           SU= ");  Serial.print(MOZStdU); Serial.print(":"); Serial.println(SUMinute);

        // ** von int zu String umkonvertieren **
        String sas = String(MOZStd);   // SA-Stunde
        String sam = String(SAMinute); // SA-Minute
        String sus = String(MOZStdU);  // SU-Stunde
        String sum = String(SUMinute); // SU-Minute

        if (SAMinute < 10)  // vorlaufende 0 bei den Minuten, sonst aber immer 1-stlg. Stunde
        {
          myGLCD.print("S-Aufgang:  ",  34, 120); myGLCD.print(sas,  250, 120); myGLCD.print(":0", 265, 120); myGLCD.print(sam,  297, 120);
        }
        else
        {
          myGLCD.print("S-Aufgang:  ",  34, 120); myGLCD.print(sas,  250, 120); myGLCD.print(":",  265, 120); myGLCD.print(sam,  280, 120);
        }

        if (SUMinute < 10)  // vorlaufende 0 bei den Minuten, sonst aber immer 2-stlg. Stunde
        {
          myGLCD.print("S-Untergang:",  34, 140); myGLCD.print(sus,  230, 140); myGLCD.print(":0", 265, 140); myGLCD.print(sum,  297, 140);
        }
        else
        {
          myGLCD.print("S-Untergang:",  34, 140); myGLCD.print(sus,  230, 140); myGLCD.print(":",  265, 140); myGLCD.print(sum,  280, 140);
        }


        // *** Ausgabe des Sonnenwinkel ** sofern die Sonne noch nicht untergegangenen ist ******************************
        if (SH1 < 0)
        { Serial.println(" Die Sonne ist bereits untergegangen. Daher erfolgt keine Ausgabe der Sonnenhoehe.");
          myGLCD.print("Sonnenhoehe:",  34, 160);
          myGLCD.print("> SU", 233, 160);
          myGLCD.print("Azimut:",  34, 180);
          myGLCD.print(" > SU", 217, 180);
          myGLCD.setFont(SmallFont); myGLCD.print("", 300, 158); myGLCD.setFont(BigFont); // kein Grad-Zeichen
        }
        else
        { // Serial.print("sin(Sonnenhoehe)= "); Serial.print(SH4); Serial.print("  .. und in Grad: "); Serial.println(SH1);
          Serial.print("  Sonnenhoehe=  "); Serial.println(SH1);
          Serial.print("Azimut-Winkel= ");  Serial.println(AZ);
          myGLCD.setColor(VGA_WHITE);           // Schriftfarbe

          char sh1[10];
          dtostrf(SH1, 2, 1, sh1);
          myGLCD.print("Sonnenhoehe:",  34, 160);
          myGLCD.print(sh1, 233, 160);
          myGLCD.setFont(SmallFont); myGLCD.print("0", 300, 158); myGLCD.setFont(BigFont); // Nur fuer das Grad-Zeichen - eine '0' im kleinsten Schriftsatz

          char az1[10];
          dtostrf(AZ, 2, 1, az1);
          myGLCD.print("Azimut:",  34, 180);
          myGLCD.print(az1, 217, 180);
          myGLCD.setFont(SmallFont); myGLCD.print("0", 300, 178); myGLCD.setFont(BigFont); // Nur fuer das Grad-Zeichen - eine '0' im kleinsten Schriftsatz
        } // ** Ende else SH1 < 0 **

        if (TgMM < 10) // vorlaufende '0' beruecksichtigen
        {
          Serial.print("Hoechststand um "); Serial.print(TgMS); Serial.print(":0"); Serial.println(TgMM);
        }
        else
        {
          Serial.print("Der Hoechststand ist um "); Serial.print(TgMS); Serial.print(":");  Serial.println(TgMM);
        }
      }// ** Ende else bs <> 0 ***


      //  ****** Display- und serielle Ausgaben ***************************************************************************************************
      Serial.print("Die Mittagshoehe betraegt= "); Serial.println(SHm);
      Serial.print("Heute ist der ");   Serial.print(result);  Serial.println(". Tag des Jahres."); Serial.println("");

      // *** Tageslaenge ** Ausgabe ****************************
      Serial.print("  Tageslaenge= ");  Serial.print(TgLS);    Serial.print(":"); Serial.println(TgLM);
      if (TgLM < 10)  // vorlaufende '0' bei der Minute-Tageslaenge
      {
        myGLCD.print("Tageslaenge: ", 35, 20); myGLCD.print(":0", 260, 20);  myGLCD.print(String(TgLM), 293, 20);
      }
      else
      {
        myGLCD.print("Tageslaenge: ", 35, 20); myGLCD.print(":", 260, 20);  myGLCD.print(String(TgLM), 275, 20);
      }

      if (TgLS < 10)  // vorlaufende '0' bei der Stunde-Tageslaenge
      {
        myGLCD.print(String(TgLS), 242, 20);
      }
      else
      {
        myGLCD.print(String(TgLS), 226, 20);
      }


      // *** Mittagszeit & Hoehe ** Ausgabe ******************************
      if (TgMM < 10) // vorlaufende '0' bei der Mittagszeit
      {
        myGLCD.print("Mittag: ", 35, 40); myGLCD.print(String(TgMS), 149, 40); myGLCD.print(":0", 185, 40); myGLCD.print(String(TgMM), 218, 40);
      }
      else
      {
        myGLCD.print("Mittag: ", 35, 40); myGLCD.print(String(TgMS), 149, 40); myGLCD.print(":", 185, 40); myGLCD.print(String(TgMM), 202, 40);
      }
      myGLCD.print("/", 242, 40); myGLCD.print(String(SHm),  265, 40);
      myGLCD.setFont(SmallFont); myGLCD.print("0", 345, 38); myGLCD.setFont(BigFont); // Nur fuer das Grad-Zeichen - eine '0' im kleinsten Schriftsatz


      // *** Im Folgenden werden die empfangenen Pruefdaten dargestellt ************
      // myGLCD.setColor(VGA_GREEN);          // Schriftfarbe: grün
      // myGLCD.setFont(SmallFont);
      // myGLCD.print("EGNOS ", 100, 20);      // EGNOS sendet 0=kein, 1=ohne, 2=OK
      // myGLCD.print(getparam(6), 180, 20);
      // Serial.print("EGNOS= "); Serial.println(getparam(6));
      // myGLCD.print(getparam(7), 10, 20);   // Anzahl Satelliten im Zugriff
      // myGLCD.print("Sat", 45, 20);

      // Serial.print("Aktuell befinden sich "); Serial.print(getparam(7)); Serial.println(" Satelliten im Zugriff.");



      // ****** Kennzeichen-Feld für GPS, sowie SAT oder RTC erstellen *******
      if (bs == 0)  // Wenn die Variable bs (Bogensekunde) = '0', bzw. leer ist, ist noch kein GPS-Empfang aktiv
      {
        myGLCD.setColor(122, 122, 122);      // grau - Balkenfarbe
        myGLCD.fillRect(690, 390, 780, 395); // x1, y1, x2, y2
        Serial.println("kein Positions-Empfang");
      }
      else
      { // Ausreichendes GPS-Signal fuer Position ist vorhanden
        myGLCD.setColor(124, 252, 0);        // gruen - untere Zeitanzeige --> http://www.farb-tabelle.de/de/farbtabelle.htm#green
        myGLCD.setFont(BigFont);
        myGLCD.print("P o s", 695, 411);
      }

      if (bs == 0)  // Wenn kein SAT-Empfang der UTC vorhanden ist ...
      {
        myGLCD.setColor(255, 215, 0);        // gelb - untere Zeitanzeige --> http://www.farb-tabelle.de/de/farbtabelle.htm#green
        myGLCD.setFont(BigFont);
        myGLCD.print("R T C", 695, 435);
      }

      else
      { // *** SD-Karte bei Bedarf mit Synchronisations-Wert beschreiben *** und RTC synchronisieren *************
        // *** Pruefung ob Synchronisation noetig *** Pruefung kann nur mit vorhandenem GPS-Signal erfolgen !! ***
        // *** Alternativ muesste ansonsten die Julianus-Zahl auch auf Basis der RTC-Zeit ermittelt werden ! *****
        if ((result - ZAHL) < 25 && (result - ZAHL) >= 0)
        { // *** wenn die aktuelle Julianus-Zahl abzgl. der hinterlegten Zahl < 25 ist und >= 0 ist (also : keine Minus-Zahl) ... nichts weiter machen
          Serial.print("Julianus= "); Serial.print(result); Serial.print(" gespeicherte ZAHL= "); Serial.println(ZAHL); // nur Kontrolle
          Serial.print("RTC-Synchronisation nicht noetig. Der Wert lautet: "); Serial.println(result - ZAHL);           // nur Kontrolle
          myGLCD.setColor(124, 252, 0);     // diese 3 Zeilen kommen spaeter auch in einen darunter liegenden Bereich, sofern erfolgreich synchronisiert wurde
          myGLCD.setFont(BigFont);
          myGLCD.print("sync", 680, 370); myGLCD.print(String(result - ZAHL), 750, 370); // Sync-Hinweis-Feld und Zahl der letzten Synchro
        }
        else
        { // *** ansonsten wird die GPS-Zeit zur Uebergabe vorbereitet werden. ***
          rtc.adjust(DateTime(year(), month(), day(), hour(), minute(), (second() + 1))); // Hier wird die RTC auf die GPS-Zeit +1 Sekunde gesetzt
          Serial.println("RTC wurde synchronisiert !");
          myGLCD.setColor(124, 252, 0);
          myGLCD.setFont(BigFont);

          // Nachdem nun die RTC synchronisiert wurde, wird jetzt die gespeicherte Zahl auf der SD-Karte aktualisiert
          Serial.print("Julianus= "); Serial.print(result); Serial.print(" gespeicherte ZAHL= "); Serial.println(ZAHL); // zur Kontrolle
          Serial.print("Es muss synchronisiert werden. Der Wert lautet: "); Serial.println(result - ZAHL);              // zur Kontrolle
          sd.remove("sync.txt");                       // loeschen der bisherigen 'sync.txt'
          syncFile = sd.open("sync.txt", FILE_WRITE);  // neue 'sync.txt' erstellen
          if (syncFile)                                // und in die geoeffnete Datei schreiben
          {
            syncFile.println(result);
            syncFile.close();                          // schliesse die Datei wieder
            myGLCD.print("sync", 680, 370); myGLCD.print(String(result - ZAHL), 750, 370); // Sync-Hinweis-Feld und Zahl der letzten Synchro
          }
          else
          { // ansonsten gebe eine Fehlermeldung aus
            Serial.println("Fehler beim Oeffnen der Datei sync.txt");
          } // else (syncFile)
        } // else (result - ZAHL < 25)
      } // else (Signal == 0)
    } // ** Ende Minute **

    // **************** R T C ********************************************************
    DateTime now = rtc.now();
    if (now.hour() < 10)  // Wenn Stunde-Zahl kleiner 10 ist, dann eine '0' davor setzen
    {
      myGLCD.setColor(255, 215, 0); myGLCD.setFont(BigFont); myGLCD.print("0", 676, 350); myGLCD.print(String(now.hour()), 691, 350); // RTC-Stunde
    }
    else
    {
      myGLCD.setColor(255, 215, 0); myGLCD.setFont(BigFont); myGLCD.print(String(now.hour()), 676, 350); // RTC-Stunde
    }

    if (now.minute() < 10)  // Wenn Minute kleiner 10 ist, dann eine '0' davor setzen
    {
      myGLCD.setColor(255, 215, 0); myGLCD.setFont(BigFont); myGLCD.print("0", 715, 350); myGLCD.print(String(now.minute()), 730, 350); // RTC-Minute
    }
    else
    {
      myGLCD.setColor(255, 215, 0); myGLCD.setFont(BigFont); myGLCD.print(String(now.minute()), 715, 350); // RTC-Minute
    }

    if (now.second() < 10)   // Wenn die Sekunde kleiner als 10 ist, dann eine '0' vorausstellen
    {
      myGLCD.setColor(255, 215, 0); myGLCD.setFont(BigFont); myGLCD.print("0", 752, 350); myGLCD.print(String(now.second()), 767, 350); // RTC-Sekunde
    }
    else
    {
      myGLCD.setColor(255, 215, 0); myGLCD.setFont(BigFont); myGLCD.print(String(now.second()), 752, 350);   // RTC-Sekunde
    } // else (now.second)


    // ************ ersatzweise RTC-Haupt-Anzeige, solange noch keine GPS-Zeit zur Verfuegung steht ****
    if (year() < 2018) // das ist ein Indiz fuer einen fehlenden Zeitsignal-Empfang
    {
      myGLCD.setBackColor(122, 122, 122); // Hintergrundfarbe der Anzeige-Bereiche
      myGLCD.setFont(SevenSeg_XXXL_Num);  // Uebergrosse Anzeige
      DateTime now = rtc.now();

      // ******** Stunde ***** RTC *** Hauptanzeige ******* --> Basis-Positionierung wie in Zeilen 903 ff.
      // ******** Zeile ist immer '340' !!! Abstand von einstelliger Zahl zu '0' ist 63 Pixel *******
      if (now.hour() < 10)  // Wenn Stunde-Zahl kleiner als 10 ist, dann eine '0' davor setzen
      {
        myGLCD.print("0", 170, 340); myGLCD.print(String(now.hour()), 233, 340); // RTC-Stunde
      }
      else
      {
        myGLCD.print(String(now.hour()),   170, 340);        // Stunde-local
      }

      // ********* Minute ***** RTC *************
      if (now.minute() < 10)  // Wenn Minute kleiner als 10 ist, dann eine '0' davor setzen
      {
        myGLCD.print("0", 330, 340); myGLCD.print(String(now.minute()), 393, 340);    // vorl. '0' + RTC-Minute
      }
      else
      {
        myGLCD.print(String(now.minute()), 330, 340);                                // Minute-local, X_Achse, Y-Achse
      }

      // ********* Sekunde ***** RTC *************
      if (now.second() < 10)   // Wenn die Sekunde kleiner als 10 ist, dann eine '0' vorausstellen
      {
        myGLCD.print("0", 490, 340); myGLCD.print(String(now.second()), 553, 340);    // Sekunde-local, X-Achse, Y-Achse // vorl. '0' + RTC-Sekunde
      }
      else
      {
        myGLCD.print(String(now.second()), 490, 340);                                 // Sekunde-local, X-Achse, Y-Achse // RTC-Sekunde
      }

      // ******* Doppelpunkte *******************
      myGLCD.setFont(BigFont);
      myGLCD.print(":", 310, 390);        // Doppelpunkt(UTC), Spalte, Zeile  war 196, 320
      myGLCD.print(":", 470, 390);        // Doppelpunkt, war 271, 320

      // *** Zuruecksetzen der Schriftgroesse und -farbe ***
      myGLCD.setFont(BigFont);
      myGLCD.setColor(124, 252, 0);
    }


    // **************************************************************************************************************+

    // ******* S E K U N D E *****************
    os = second();
    char a[2];
    sprintf(a, "%02d", second());
    sprintf(std, "%02d", hour());         // Stunde-local
    sprintf(m, "%02d", minute());

    if (year() >= 2018) // das ist ein Indiz fuer einen korrekten Zeitsignal-Empfang
    {
      myGLCD.setColor(124, 252, 0);           // gruen - untere Zeitanzeige --> http://www.farb-tabelle.de/de/farbtabelle.htm#green
      myGLCD.setFont(BigFont);
      myGLCD.print("S A T", 695, 435);
      // **** Hier wird nun die GPS-Zeit zur Anzeige gebracht ****
      myGLCD.setBackColor(122, 122, 122); // Hintergrundfarbe der Anzeige-Bereiche
      myGLCD.setColor(0, 255, 0);         // gruen
      myGLCD.setFont(SevenSeg_XXXL_Num);  // Uebergrosse Anzeige
      // ******** Stunde & Minute ***************
      myGLCD.print(a,  490, 340);         // Sekunde-local, X-Achse, Y-Achse
      myGLCD.print(std, 170, 340);        // Stunde-local
      myGLCD.print(m,  330, 340);         // Minute-local, X_Achse, Y-Achse
      // ******* Doppelpunkte *******************
      myGLCD.setFont(BigFont);
      myGLCD.print(":", 310, 390);        // Doppelpunkt(UTC), Spalte, Zeile  war 196, 320
      myGLCD.print(":", 470, 390);        // Doppelpunkt, war 271, 320
    }
    else
    { // nichts tun
    } // Ende year >= 2018

    // **** MondPhase *** Jede Stunde, immer zur 16.Minute || nach Neustart aufrufen ****
    if (now.minute() == 16 || moon == 0)
    {
      mondPhase(); // ** Aufruf der Mondphasen-Berechnung und -Anzeige **
    }
    else
    { // nichts tun
    }  // ** Ende MondPhase **


    // Nur einmal die Stunde und zu Beginn *** siehe dazu Mond ***
    if (now.minute() == 20 && now.second() == 20 || dastr == 0)
    { // *** Aufbau LOG-Datei *** Erstellen eines Datensatzes ****
      dataString += (now.day());
      dataString += ";";
      dataString += (now.month());
      dataString += ";";
      dataString += (now.year());
      dataString += "; ";
      dataString += (now.hour());
      dataString += ";";
      dataString += (now.minute());
      dataString += "; ";
      dataString += (b1); // Thermo
      dataString += ";";
      dataString += "C; ";
      dataString += (l1); // Hygro
      dataString += ";";
      dataString += "%; ";
      dataString += (ba); // Baro
      dataString += ";";
      dataString += "hPa";
      dataString += "; ";
      dataString += (getparam(2));   // Koordinaten
      dataString += ";";
      dataString += (getparam(3));   // Koordinaten
      dataString += ";";
      dataString += (getparam(4));   // Koordinaten
      dataString += ";";
      dataString += (getparam(5));   // Koordinaten
      dataString += ";";
      dataString += (Loc1); (Loc2); (Loc5); (Loc6); (Loc3); (Loc4);             // Locator 1-2-5-6-3-4
      dataString += ";  ";
      if (SAMinute < 10)
      {
        dataString += (MOZStd); dataString += ":0"; dataString += (SAMinute);   // Sonnenaufgang mit vorl. 0
      }
      else
      {
        dataString += (MOZStd); dataString += ":"; dataString += (SAMinute);    // Sonnenaufgang
      }
      dataString += "; ";
      if (SAMinute < 10)
      {
        dataString += (MOZStdU); dataString += ":0"; dataString += (SUMinute);  // Sonnenuntergang mit vorl. 0
      }
      else
      {
        dataString += (MOZStdU); dataString += ":"; dataString += (SUMinute);   // Sonnenuntergang
      }
      dataString += "; ";
      dataString += (TgMS); dataString += ":"; dataString += (TgMM);         // Mittagszeit
      dataString += "; ";
      dataString += (SHm);                                                   // Mittags-Sonnenhoehe
      dataString += "; ";
      dataString += (SH1);                                                   // aktuelle Sonnenhoehe
      dataString += "; ";
      dataString += (AZ);                                                    // aktueller Seiten-Winkel (Azimut)
      dataString += "; ";
      if (TgLM < 10)
      {
        dataString += (TgLS); dataString += ":0"; dataString += (TgLM);      // Tageslaenge mit vorl. 0 Minute
      }
      else
      {
        dataString += (TgLS); dataString += ":"; dataString += (TgLM);       // Tageslaenge
      }
      dataString += "; ";
      Serial.println("Tag; Monat; Jahr; Stunde; Minute; Thermo; Hygro; Baro; Koordinaten;; SA, SU; Mittagszeit;; Mittagshoehe; S-Hoehe, S-Winkel; Tageslaenge");
      Serial.println(dataString); Serial.println(" ");
      dastr = 1;                                   // diese Variable dient der nur einmaligen Aktualisierung nach dem Start

      // *** Speichern der Sensor-Daten *****
      // ************************************
      // *** hier werden die Kriterien ueberprueft, welche erfuellt sein muessen, um einen Schreib-Vorgang zu starten ****
      if (year() < 2019) // *** 2. ... und wenn eine korrekt GPS-Zeit vorhanden ist
      {
        Serial.println(); Serial.println(" Kein GPS-Signal - keine Speicherung "); // .. nichts machen
        // *** Default-Werte für die Ampel-Darstellung ***
        myGLCD.setColor(180, 0, 0);                    // rot
        myGLCD.fillRect(725, 397, 760, 405);           // links,oben,rechts,unten  -->  x1, y1, x2, y2
      }
      else
      { // ... ansonsten weitermachen
        Serial.print(" GPS-Signal vorhanden. Speicherung moeglich.  year= "); Serial.println(year());
        myGLCD.setColor(180, 0 , 0);                   // Feldfarbe: rot
        myGLCD.fillRect(725, 397, 760, 405);           // links,oben,rechts,unten  -->  x1, y1, x2, y2
        logDat = sd.open("tft9.txt", FILE_WRITE);      // Oeffenen der Datei 'tft9.txt' zum nachfolgenden SCHREIBEN
        Serial.println("tft9.txt zum Schreiben oeffnen.");
        if (logDat)
        {
          logDat.println(dataString);                  // Beschreiben der soeben erstellten Datei
          logDat.close();                              // Datei nach dem Schreiben wieder schliessen
          Serial.println("Speicherung erfolgreich. "); // Serial.print("Die Variable go wird zurueckgesetzt auf den Wert: "); Serial.println(go);
          dataString = "";                             // *** DatenString loeschen *****
          // *** Ampel ***
          myGLCD.setColor(0, 180, 0);                  // gruen
          myGLCD.fillRect(725, 397, 760, 405);         // links,oben,rechts,unten  -->  x1, y1, x2, y2

        }
        else
        {
          Serial.println("Fehler beim Oeffnen der Datei tft9.txt !"); // Hinweis fuer den Fall, dass sich die Datei nicht mehr oeffnen laesst
        } // Ende Schreibfehler
      } // Ende GPS-Signal ?
    }
    else
    {
      // nichts tun
    } // *** Ende Datastring
  } // *** Ende Sekunde ***
}  // *** Ende void L O O P ***
// *****************************************************************************************************+


// *** Titel-Bild *************
// ****************************
void screenHome()  // draw main home screen showing overview info
{
  myGLCD.clrScr(); // clear the screen
  myGLCD.setColor(255, 255, 255);
  int mytime = millis();
  // myFiles.load(0, 0, 800, 480, "qmk804.raw"); // Quermarkenfeuer, 800x480 Pixel-Foto laden
  // Foto = "qmk804.raw";
  myFiles.load(0, 0, 800, 480, "PIC805.raw"); // Planet, 800x480 Pixel-Foto laden
  Foto = "PIC805.raw";
  Serial.println(millis() - mytime);
} // ** Ende void() screenHome **


// *** Blinker ****************
// ****************************
// ** Lass mal blinken **  Abhaengigkeit und Positionierung des Blinkens vom Foto ****
// Es wird kontrolliert, ob die Zeit für das Blink-Intervall schon abgelaufen ist. Wenn die Zeitdifferenz zwischen dem
// letzten Abspeichern und der aktuellen Zeit groeßer als das Intervall ist, so wird die nachfolgende Funktion ausgeführt.
void blinker()
{
  if (Foto == "qmk804.raw")  // nur beim 'Quermarkenfeuer'
  {
    if (value == LOW)
    { //     (Spalte, Zeile, Breite, Hoehe, Farbe);
      myGLCD.setColor(54, 54, 54);           // grau  --> Farbtabelle : http://www.farb-tabelle.de/de/farbtabelle.htm
      myGLCD.fillRect(277, 231, 287, 240);   // (Spalte, Zeile, Breite, Hoehe);
      myGLCD.setColor(66, 66, 66);
      myGLCD.fillRect(288, 231, 297, 240);
    }
    else
    {
      myGLCD.setColor(127, 255, 0);          // chartreuse
      myGLCD.fillRect(277, 231, 287, 240);
      myGLCD.setColor(255, 69, 0);           // firebrick1
      myGLCD.fillRect(287, 231, 297, 240);
    }

    // Serial.print("millis= "); Serial.print(millis()); Serial.print(" previous= "); Serial.println(previousMillis);
    if (millis() - previousMillis > interval)
    {
      previousMillis = millis();             // aktuelle Zeit abspeichern
      value = !value;                        // Zustand wechseln
    } // ** Ende if millis **
  } // ** Ende if Foto **
} // ** Ende Blinken **



void mondPhase() // **** Mond-Bild einblenden *** und Phasen-Berechnung *** und Sichtbarkeits-Berechnung ************
{
  DateTime now = rtc.now();
  myGLCD.setColor(255, 255, 255); myGLCD.setFont(BigFont); // weiss oder gruen --> myGLCD.setColor(124, 252, 0); myGLCD.setFont(BigFont);
  double Umlauf = 29.53058886;              // synodischer Monat in Tagen
  double URSPRUNG = 1548047794;             // 21.01.2019, 06:16:34 als UNIX-Zeit
  double UNIX2  =  now.unixtime();          // aktuelle Zeit als UNIX-Zeit
  float Anz1    = (UNIX2 - URSPRUNG);       // Anzahl Sekunden seit dem 21.01.2019
  double Anz2   = Anz1 / 86400;             // Anzahl Tage seit dem 21.01.2019 als Nachkomma-Zahl
  double Phase1 = (Anz2 / Umlauf);          // Anzahl Umlaeufe seit dem 21.01.2019
  int Anz4      = Phase1;                   // Umlaeufe als Ganzzahl
  float Anz3    = (Phase1 - Anz4);          // Umlaeufe - nur Nachkomma-Wert
  int Phase     = (Anz3 * 100);             // aktuelle Phase seit letztem Vollmond
  moon = 1;

  /*    Serial.print ("unix1 (21.01.2019)= "); Serial.println (unix1);                      // die Unix-Zeit vom 21.01.2019, 06:16:34       = 1548047794
      Serial.print ("unix2 (aktuelle Zeit) = "); Serial.println (now.unixtime());         // die aktuelle Unix-Zeit vom 01.02.2019, 17:30 = 1549041897
      Serial.print ("Sekunden seit 21.01.2019 = "); Serial.println (Anz1);                // 994103
      Serial.print ("Anzahl Tage seit dem 21.01.2019 = "); Serial.println (Anz2);         // 994103 / 86400 Sekunden = 11,535821759259...
      Serial.print ("Umlaeufe seit Vollmond - Ganzzahl = "); Serial.println (Anz4);       // 11,54
      Serial.print ("... nur Nachkomma-Stellen = "); Serial.println (Anz3);               // 0,54
      Serial.print ("Phase in % = "); Serial.println (Phase);                             // ausgehend vom letzten Vollmond !!!
  */

  // Bei 8 Bildern : zun. 25%, zun. 50%, zun. 75%, 100% (Vollmond), abn. 75%, abn. 50%, abn. 75%, 0% (Neumond)
  if (Phase > 32 && Phase <= 44)            // abnehmend 1/4 Mond = 37,5%
  {
    myFiles.load(480, 30, 62, 62, "3_Mond.raw"); Foto = "3_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else if (Phase > 44 && Phase <= 56)       // Neumond = 50%%
  {
    myFiles.load(480, 30, 62, 62, "4_Mond.raw"); Foto = "4_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else if (Phase > 56 && Phase <= 69)        // zunehmend 1/4 Mond = 62,5%
  {
    myFiles.load(480, 30, 62, 62, "5_Mond.raw"); Foto = "5_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else if (Phase > 69 && Phase <= 82)        // zunehmend 1/2 Mond = 75,0%
  {
    myFiles.load(480, 30, 62, 62, "6_Mond.raw"); Foto = "6_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else if (Phase > 82 && Phase <= 95)        // zunehmend 3/4 Mond = 87,5%
  {
    myFiles.load(480, 30, 62, 62, "7_Mond.raw"); Foto = "7_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else if (Phase > 6 && Phase <= 19)         // abnehmend 3/4 Mond = 12,5%
  {
    myFiles.load(480, 30, 62, 62, "1_Mond.raw"); Foto = "1_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else if (Phase > 19 && Phase <= 32)        // abnehmend 1/2 Mond = 25,0%
  {
    myFiles.load(480, 30, 62, 62, "2_Mond.raw"); Foto = "2_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }
  else                                       // Vollmond = 100%, bzw. 0%
  {
    myFiles.load(480, 30, 62, 62, "0_Mond.raw"); Foto = "0_Mond.raw";  // X-Achse, Y-Achse, Bildgroesse, Bildgroesse, "Bildname")
  }

  // *** Es folgt die Ausgabe der aktuellen Mond-Phase, sowie deren prozentuale Sichtbarkeit und dem Zusatz 'abnehmend', oder 'zunehmend'
  if (Phase < 50)
  {
    myGLCD.print("- abnehmend",  555, 95);
    int Sichtbarkeit = (100 - (Phase * 2));
    if (Sichtbarkeit < 10) // wenn Wert einstellig
    {
      myGLCD.print("  ", 482, 95);  myGLCD.print(String(Sichtbarkeit), 497, 95); myGLCD.print("% ", 512, 95);
    }
    else
    {
      myGLCD.print(String(Sichtbarkeit), 482, 95); myGLCD.print("% ", 512, 95);
    }

  }
  else
  {
    myGLCD.print("- zunehmend",  555, 95);
    int Sichtbarkeit = ((Phase - 50) * 2);
    if (Sichtbarkeit < 10) // wenn Wert einstellig
    {
      myGLCD.print("  ", 482, 95);  myGLCD.print(String(Sichtbarkeit), 497, 95); myGLCD.print("% ", 512, 95);
    }
    else
    {
      myGLCD.print(String(Sichtbarkeit), 482, 95); myGLCD.print("% ", 512, 95);
    }
  }
  // Serial.println("Aktualisierung der Monddaten."); myGLCD.print("Mondphase=",  480, 95);  myGLCD.print(String(Phase), 640, 95); myGLCD.print("%",  670, 95);
} // **** Ende void(mondphase) ****



// *** Sensorik ***************
// ****************************
void screenSens()  // Anzeige der Sensorwerte
{
  myGLCD.setBackColor(122, 122, 122);// Hintergrundfarbe der Anzeige-Bereiche
  myGLCD.setColor(0, 255, 0);        // Schriftfarbe
  myGLCD.setFont(BigFont);           // Schriftgroesse

  // *** Baro ***
  float d = bmp.readPressure();      // Luftdruck
  float baro = (d + 745) / 100;      // Korrektur auf den hiesigen aktuellen Wert
  // char ba[10];
  dtostrf(baro, 4, 1, ba);           // von float zu string (umzuwandelnde Variable, Vorkommastellen, Nachkommastellen, Ziel-Variable)
  myGLCD.print("Baro" , 570, 70);    // Text, Zeile 80
  myGLCD.print (ba, 645, 70);        // Baro-Wert
  myGLCD.print ("hPa", 745, 70);     // Einheit
  Serial.print("Baro: "); Serial.println(ba);
  // Serial.print("Baro: "); Serial.println(bmp.readPressure());

  // *** Hygro ***
  float hygro = dht.readHumidity();  // Luftfeuchte auslesen, sowie mit einem Korrektur-Wert addieren
  // char l1[10];
  dtostrf(hygro, 2, 1, l1); // Alternativ-Befehl, da Programmfehler bei Umwandlung von float, oder double Variablen zu String **** Float-Variable,2 Vorkommastellen, 2 Nachkommastellen, String-Variable
  myGLCD.print("Hygro", 570,  45);   // Text, Zeile 50
  myGLCD.print (l1, 680, 45);        // Hygro-Wert
  myGLCD.print ("%", 745, 45);       // Einheit
  Serial.print("Hygro: "); Serial.println(dht.readHumidity());

  // *** Thermo ***
  // float temp  = dht.readTemperature(); // dht-Temperatur auslesen
  float temp  = bmp.readTemperature(); // bmp-Temperatur auslesen
  // char b1[10];
  dtostrf(temp, 2, 1, b1);
  myGLCD.print("Temp", 570,  20);    // Text, Zeile 20
  myGLCD.print (b1, 680, 20);        // Thermo-Wert
  myGLCD.print ("C", 745, 20);       // Einheit
  Serial.print("Thermo: "); Serial.println(bmp.readTemperature()); Serial.println("");
} // ** Ende void() screenSens **




// *** Zeitzeichen-Signal ***
void beep()                          // DLF-Zeitzeichen-Laenge
{
  int m = minute(); int s = second();
#define PIPS    (TICKPIN,1000,150) // 5 Sek. vor der vollen Stunde
#define BEEP    (TICKPIN,1000,850) // Stundensignal
#define TICK    (TICKPIN,1000,1)   // Sekunden-Tick
  boolean mpips = m == 29 || m == 59;
  boolean spips = s == 57 || s == 58 || s == 59 || s == 55 || s == 56;
  boolean sbeep = m == 0 && s == 0;
  boolean pips = mpips && spips;
  if (sbeep)tone BEEP; else if (pips)tone PIPS; else tone TICK;
  // if(sbeep) Serial.print("BEEP ");else if(pips) Serial.print("PIPS "); else Serial.print("Tick ");
}


/* ***************************************************************************************************
 * ****** Tageszahl für Sonnenlauf ermitteln *********************************************************
   Pruefe ggf. ob es sich um eine Schaltjahr handelt - das kann etwas verwirrend sein
   See: https://support.microsoft.com/en-us/kb/214019 ********************************************** */

int calculateDayOfYear(int day, int month, int year)
{
  // Given a day, month, and year (4 digit), returns the day of year. Errors return 999
  int daysInMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

  // Verify we got a 4-digit year
  if (year < 1000)
  {
    return 999;
  }

  if (year % 4  == 0)
  {
    if (year % 100 != 0)
    {
      daysInMonth[1] = 29;
    }
    else
    {
      if (year % 400 == 0)
      {
        daysInMonth[1] = 29;
      }
    }
  }

  // Make sure we are on a valid day of the month
  if (day < 1)
  {
    return 999;
  } else if (day > daysInMonth[month - 1])
  {
    return 999;
  }

  int doy = 0;
  for (int i = 0; i < month - 1; i++)
  {
    doy += daysInMonth[i];
  }

  doy += day;
  return doy;
} // ****** Ende Tageszahlbestimmung *************************************

int sommerzeit(DateTime ls_now)
{
  int lv_aenderungsstunde = 2; // Aenderungszeit 2 Uhr
  int lv_day_sonntag;
  // Sommerzeit letzter Sonntag Maerz   2 Uhr 25. - 31.
  //        bis letzter Sonntag Oktober 2 Uhr 25. - 31.
  if (ls_now.month()  < 3 || ls_now.month() > 10  || (ls_now.month() ==  3 && ls_now.day() < 25)) return 0; // wenn kleiner als 25.03.
  if ((ls_now.month() > 3 && ls_now.month() < 10) || (ls_now.month() == 10 && ls_now.day() < 25)) return 1; // wenn kleiner als 25.10.
  if (ls_now.month() ==  3 && ls_now.dayOfTheWeek() == 0 && ls_now.hour() >= lv_aenderungsstunde) return 1;
  if (ls_now.month() ==  3 && ls_now.dayOfTheWeek() == 0 && ls_now.hour() <  lv_aenderungsstunde) return 0;
  if (ls_now.month() == 10 && ls_now.dayOfTheWeek() == 0 && ls_now.hour() >= lv_aenderungsstunde) return 0;
  if (ls_now.month() == 10 && ls_now.dayOfTheWeek() == 0 && ls_now.hour() <  lv_aenderungsstunde) return 1;
  lv_day_sonntag = ls_now.day() - ls_now.dayOfTheWeek();
  if (ls_now.month() ==  3 && lv_day_sonntag >= 25) return 1;
  if (ls_now.month() ==  3 && lv_day_sonntag < 25) return 0;
  if (ls_now.month() == 10 && lv_day_sonntag >= 25) return 0;
  if (ls_now.month() == 10 && lv_day_sonntag < 25) return 1;
  return 0; // sollte an sich unnoetig sein
}


//---------------------------------
// *** GPS ROUTINEN
//---------------------------------

boolean getline(char *phrase) //HARD POLLING
{ char s[100]; byte b, n; unsigned long t = millis();
  for (int i = 0; i < sizeof(s); i++)s[i] = 0; Line = "";
  do
  { b = Serial1.read();  //RX1
    if (millis() > (t + 100)) {
      nogps++;
      return false;
    }
  } while (b != '$');
  s[0] = b;
  n = Serial1.readBytesUntil('\n', &s[1], 90);    // RX1
  s[n] = 0;
  if (strstr(s, phrase) == s)
  { for (int i = 0; i < n; i++)Line += s[i];
    return true;
  }
  return false;
}


// ********* G G A **********************************
void GGA()// FIX SAT ect.
{
  // int w=tft.width();
  if (minute() == 29 || minute() == 59)return;

  // LCD.setColor(VGA_WHITE);         // Schriftfarbe
  // LCD.setFont(SmallFont);
  // LCD.print(getparam(6),  20, 250);          // FIX (Synchronisation) Zeichen = 2
  // LCD.print(getparam(9),  65, 240);          // Hoehe (in meter) - 20, 240
  // LCD.print(getparam(10),130, 240);          // M --> Meter      -  ?, 240
  // LCD.print("mtr", 120, 240);                // Einheit 'fix'
  // LCD.setColor(VGA_BLACK);         // Schriftfarbe für die nächsten Ausgaben
} // * GPGGA *



// ********* R M C **********************************
void RMC() // *** Synchronisation TIME DATE *** Zerlegen des RMC-Strings ***
{ setTime(getparam(1).substring(0, 0 + 2).toInt(),
          getparam(1).substring(2, 2 + 2).toInt(),
          getparam(1).substring(4, 4 + 2).toInt(),
          getparam(9).substring(0, 0 + 2).toInt(),
          getparam(9).substring(2, 2 + 2).toInt(),
          getparam(9).substring(4, 4 + 2).toInt());
  time_t utc = (now());                 // Mit der Variablen 'utc' habe ich hier die Basis-Zeit. Dieser Wert muss aber noch umgerechnet werden !
  time_t cet = CE.toLocal(now(), &tcr); // Aufbereitung der UTC-Zeit nach der TimeZone-Regel
  setTime(cet);                         // Hier wird nun die GPS-Zeit im vorgegebenen Format in die interne Zeitbasis uebernommen



  // ****** Positionsdaten separieren ***************************************
  // ** $GNRMC,153710.00,A,5157.66892,N,00914.25091,E,2.522,331.60,041018,,,A*7B **
  // laenge = 9.2433177;   //  9°14'15.36"
  // breite = 51.961246;   // 51°57'40.28"
  // breiterichtung = 'N';
  // laengerichtung = 'E';
  Lat = (getparam(3));   // Parameter sind STRING-Variablen
  La1 = (getparam(4));   // Parameter sind STRING-Variablen
  Lon = (getparam(5));   // Parameter sind STRING-Variablen
  Lo1 = (getparam(6));   // Parameter sind STRING-Variablen

  // >>>> Werte könnten noch in eine INT-Variable umkonvertiert werden und dann durch 100 geteilt werden <<<<<


  // *** UTC-Stunde - ausgehend von der Lokalzeit und der Zeitdifferenz - ermitteln ***
  diff = (cet - utc) / 3600;     // Zeitunterschied in Stunden  ==>>> nur bei Sat-Empfang moeglich
  int ho;
  if ((hour() - diff) < 0) {
    ho = (hour() - diff) + 24;       // WENN Lokalzeit - Zeitunterschied =< 00:00, dann 24 hinzurechnen
  }
  else {
    (ho = hour() - diff);            // SONST Lokalzeit - Zeitunterschied
  }
  char c[2];
  sprintf(c, "%02d", ho);          // Ausgabe der errechneten UTC-Stunde
  UTC = ho;                        // Die Ausgabe erfolgt ziemlich weit oben
  // Serial.print("diff: "); Serial.println(diff);

  nogps = 0;


  // **** Ausgabe der UTC-Stunde ****
  myGLCD.setFont(BigFont);
  myGLCD.setColor(0, 255, 0);      // gruen
  const char* V1 = (" UTC");       // Beispiel fuer die neue, C++ konforme Schreibweise
  myGLCD.print(V1, 35, 340);       // Anzeige von UTC, Spalte, Zeile
  // myGLCD.print(" UTC", 35, 340);   // alte Schreibweise

  char u[2];
  sprintf(u, "%02d", ho);
  // myGLCD.setFont(BigFont);
  myGLCD.setFont(SevenSegNumFont);
  myGLCD.setColor(0, 255, 0);
  myGLCD.print(u, 103 , 340);      // war 135, 240

  if (UTC == 0)  // wenn keine UTC-Stunde vorhanden ist, bzw. kein GPS-Uhrzeit-Signal vorhanden ist
  {
    // Serial.print(UTC); Serial.println(" - Kein GPS");
    Signal = 0;  // Setzen der Variablen
  }
  else
  {
    // Serial.print(UTC); Serial.println(" - GPS vorhanden");
    Signal = 1;  // Setzen der Variablen
  }
} // * GPRMC *

#define hex(i)  ((i<=9) ? ('0'+i): ('A'- 10 + i))

boolean checksum()
{ byte b = 0; int e;
  e = Line.indexOf('*');
  if (e > 10)
  { for (int i = 1; i < e; i++)b ^= Line[i];
    if ((hex((b & 15)) == Line[e + 2]) &&
        (hex(b / 16) == Line[e + 1]))return true;
  }
  return false;
}


String getparam(int ix)
{ int c, cc = 0;
  if (checksum())
  { do
    { c = Line.indexOf(',', cc);
      if (c >= 0)cc = c + 1; else break;
    } while (--ix);
    return (Line.substring(c + 1, Line.indexOf(',', c + 1)));
  }
  return "xx"; //debug
}


void SerialClear()
{ while (Serial1.available())Serial1.read();   // --> TX1
}

// ************** Script E N D E *****************************
